
/*
Copyright 2007-2011 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripList(name, exclusive)
{
    this.name = "";
    this.obj = {};
    this.listLength = 0;
    this.exclusive = false;
    this.lastChanged = -1;
    this.sorted = true;

    this.setName = function(value)
    {
        if (value) this.name = "" + value;
    }

    this.setExclusive = function(value)
    {
        this.exclusive = "" + value == "true";
    }

    this.getExclusive = function()
    {
//        return this.listLength > 0 && this.exclusive;
        return !this.isEmpty() && this.exclusive;
    }

    this.isEmpty = function()
    {
        for (var k in this.obj) { return false; }
        return true;
    }

    this.__defineGetter__("length", function() {
            return this.listLength;
        });

    this.setLastChanged = function(value)
    {
        if (value) this.lastChanged = value;
    }

    this.add = function(o, purge)
    {
        if (!o) return;
        var oldObj = this.obj[o.getKey()];
        if (oldObj) {
            oldObj.merge(o);
            if (purge) oldObj.purge();
        } else {
            if (purge) o.purge();
            this.obj[o.getKey()] = o;
            this.sorted = false;
            this.listLength++;
        }
    }

    this.remove = function(o)
    {
        if (!o) return;
        this.removeByKey(o.getKey());
    }

    this.removeByKey = function(value)
    {
        if (!value || !value in this.obj) return;
        delete this.obj[value];
        this.listLength--;
    }

    this.reset = function()
    {
        for each (var o in this.obj) if (o) this.removeByKey(o.getKey());
        this.obj = {};
        this.listLength = 0;
        this.sorted = true;
    }

    this.sort = function(purge)
    {
        if (this.sorted) return;
        var a = [];
        for each (var o in this.obj) a.push(o);
        a.sort(function(a, b) { return a.compare(b); });
        this.reset();
        for (var i = 0; i < a.length; i++) this.add(a[i], purge);
        this.sorted = true;
    }

    this.purge = function()
    {
        for each (var o in this.obj) if (o) o.purge();
    }

    this.clone = function(purge)
    {
        var tmp = new this.constructor(this.name, this.exclusive);
        for each (var o in this.obj) if (o) tmp.add(o.clone(purge));
        return tmp;
    }

    this.merge = function(list)
    {
        this.setExclusive(this.getExclusive() || list.getExclusive());
        for each (var o in list.obj) this.add(o.clone());
    }

    this.subtract = function(list)
    {
        this.setExclusive(this.getExclusive() && !list.getExclusive());
        for (var k in this.obj) if (k in list.obj) this.removeByKey(k);
    }

    this.generateXml = function()
    {
        var tmp = "";
        for each (var o in this.obj) if (o) tmp += o.generateXml();
        if (tmp != "") {
            return "\t\t\t" +
                "<" + this.name + (this.exclusive ? " exclusive=\"" + this.exclusive + "\"" : "") + ">\n" +
                tmp +
                "\t\t\t</" + this.name + ">\n";
        } else {
            return "";
        }
    }

    this.setName(name);
    this.setExclusive(exclusive);
}
YaripList.prototype = new YaripObject;
YaripList.prototype.constructor = YaripList;

function YaripElementList(name)
{
    this.name = "";
    this.obj = {};
    this.listLength = 0;
    this.sorted = true;

    this.setName(name);
}
YaripElementList.prototype = new YaripList;
YaripElementList.prototype.constructor = YaripElementList;

function YaripElementWhitelist(name)
{
    this.name = "whitelist";
    this.obj = {};
    this.listLength = 0;
    this.sorted = true;

    this.setName(name);
}
YaripElementWhitelist.prototype = new YaripElementList;
YaripElementWhitelist.prototype.constructor = YaripElementWhitelist;
YaripElementWhitelist.prototype.set = function(row, col, value)
{
    if (value in this.obj) return;

    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        switch (col) {
        case 0:
            if (!this.checkXPath(value)) return false;

            var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
            fh.removeEntry("xpath", o.getXPath());
            this.remove(o);

            o.setXPath(value);
            this.add(o);
            fh.addEntry("xpath", value);
            return true;
        case 1:
            o.setDeep(value);
            return false;
        case 2:
            o.setForce(value);
            return false;
        default:
            return false;
        }
    }
    return false;
}
YaripElementWhitelist.prototype.get = function(row, col)
{
    if (!this.sorted) this.sort();
    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        switch (col) {
        case -1: return o.getKey();
        case 0: return o.getXPath();
        case 1: return o.getDeep();
        case 2: return o.getForce();
        case 3:
            var ms = o.getCreated();
            if (ms > -1) {
                var date = new Date(ms);
                return date.toDateString() + " " + date.toLocaleTimeString();
            } else {
                return "";
            }
        case 4:
            var ms = o.getLastFound();
            if (ms > -1) {
                var date = new Date(ms);
                return date.toDateString() + " " + date.toLocaleTimeString();
            } else {
                return "";
            }
        case 5: return o.getFound();
        case 6: return o.getNotFound();
        case 7: return o.getIgnored();
        default: return "";
        }
    }
    return "";
}

function YaripElementBlacklist(name)
{
    this.name = "blacklist";
    this.obj = {};
    this.listLength = 0;
    this.sorted = true;

    this.setName(name);
}
YaripElementBlacklist.prototype = new YaripElementList;
YaripElementBlacklist.prototype.constructor = YaripElementBlacklist;
YaripElementBlacklist.prototype.set = function(row, col, value)
{
    if (value in this.obj) return;

    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        switch (col) {
        case 0:
            if (!this.checkXPath(value)) return false;

            var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
            fh.removeEntry("xpath", o.getXPath());
            this.remove(o);

            o.setXPath(value);
            this.add(o);
            fh.addEntry("xpath", value);
            return true;
        case 1:
            o.setDeep(value);
            return false;
        case 2:
            o.setForce(value);
            return false;
        case 3:
            o.setDOW(value);
            return false;
        case 4:
            o.setPlaceholder(value);
            return false;
        default:
            return false;
        }
    }
    return false;
}
YaripElementBlacklist.prototype.get = function(row, col)
{
    if (!this.sorted) this.sort();
    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        switch (col) {
        case -1: return o.getKey();
        case 0: return o.getXPath();
        case 1: return o.getDeep();
        case 2: return o.getForce();
        case 3: return o.getDOW();
        case 4: return o.getPlaceholder();
        case 5:
            var ms = o.getCreated();
            if (ms > -1) {
                var date = new Date(ms);
                return date.toDateString() + " " + date.toLocaleTimeString();
            } else {
                return "";
            }
        case 6:
            var ms = o.getLastFound();
            if (ms > -1) {
                var date = new Date(ms);
                return date.toDateString() + " " + date.toLocaleTimeString();
            } else {
                return "";
            }
        case 7: return o.getFound();
        case 8: return o.getNotFound();
        case 9: return o.getIgnored();
        default: return "";
        }
    }
    return "";
}

function YaripElementAttributeList(name)
{
    this.name = "attribute";
    this.obj = {};
    this.listLength = 0;
    this.sorted = true;

    this.setName(name);
}
YaripElementAttributeList.prototype = new YaripList;
YaripElementAttributeList.prototype.constructor = YaripElementAttributeList;
YaripElementAttributeList.prototype.set = function(row, col, value)
{
    if (this.obj[value]) return;

    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        switch (col) {
        case 0:
            if (!this.checkXPath(value)) return false;

            var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
            fh.removeEntry("xpath", o.getXPath());
            this.remove(o);

            o.setXPath(value);
            this.add(o);
            fh.addEntry("xpath", value);
            return true;
        case 1:
            var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
//            fh.removeEntry("attribute_name", o.getName());

            o.setName(value);
            fh.addEntry("attribute_name", value);
            return true;
        case 2:
            var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
//            fh.removeEntry("attribute_value", o.getValue());

            o.setValue(value);
            fh.addEntry("attribute_value", value);
            return false;
        case 3:
            o.setDeep(value);
            return false;
        default:
            return false;
        }
    }
    return false;
}
YaripElementAttributeList.prototype.get = function(row, col)
{
    if (!this.sorted) this.sort();
    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        switch (col) {
        case -1: return o.getKey();
        case 0: return o.getXPath();
        case 1: return o.getName();
        case 2: return o.getValue();
        case 3: return o.getDeep();
        case 4:
            var ms = o.getCreated();
            if (ms > -1) {
                var date = new Date(ms);
                return date.toDateString() + " " + date.toLocaleTimeString();
            } else {
                return "";
            }
        case 5:
            var ms = o.getLastFound();
            if (ms > -1) {
                var date = new Date(ms);
                return date.toDateString() + " " + date.toLocaleTimeString();
            } else {
                return "";
            }
        case 6: return o.getFound();
        case 7: return o.getNotFound();
        default: return "";
        }
    }
    return "";
}

function YaripElementScriptList(name)
{
    this.name = "script";
    this.obj = {};
    this.listLength = 0;
    this.sorted = true;

    this.setName(name);
}
YaripElementScriptList.prototype = new YaripElementList;
YaripElementScriptList.prototype.constructor = YaripElementScriptList;
YaripElementScriptList.prototype.set = function(row, col, value)
{
    if (value in this.obj) return;

    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        switch (col) {
        case 0:
            if (!this.checkXPath(value)) return false;

            var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
            fh.removeEntry("xpath", o.getXPath());
            this.remove(o);

            o.setXPath(value);
            this.add(o);
            fh.addEntry("xpath", value);
            return true;
        case 1:
            o.setDeep(value);
            return false;
        case 99:
            o.setScript(value);
            return false;
        default:
            return false;
        }
    }
    return false;
}
YaripElementScriptList.prototype.get = function(row, col)
{
    if (!this.sorted) this.sort();
    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        switch (col) {
        case -1: return o.getKey();
        case 0: return o.getXPath();
        case 1: return o.getDeep();
        case 2:
            var ms = o.getCreated();
            if (ms > -1) {
                var date = new Date(ms);
                return date.toDateString() + " " + date.toLocaleTimeString();
            } else {
                return "";
            }
        case 3:
            var ms = o.getLastFound();
            if (ms > -1) {
                var date = new Date(ms);
                return date.toDateString() + " " + date.toLocaleTimeString();
            } else {
                return "";
            }
        case 4: return o.getFound();
        case 5: return o.getNotFound();
        case 99: return o.getScript();
        default: return "";
        }
    }
    return "";
}

function YaripContentWhitelist(name, exclusive)
{
    this.name = "whitelist";
    this.obj = {};
    this.listLength = 0;
    this.exclusive = false;
    this.sorted = true;

    this.setName(name);
    this.setExclusive(exclusive);
}
YaripContentWhitelist.prototype = new YaripList;
YaripContentWhitelist.prototype.constructor = YaripContentWhitelist;
YaripContentWhitelist.prototype.set = function(row, col, value)
{
    if (value in this.obj) return;

    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        switch (col) {
        case 0:
            if (!this.checkRegExp(value)) return false;

            var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
            fh.removeEntry("regexp", o.getRegExp());
            this.remove(o);

            o.setRegExp(value);
            this.add(o);
            fh.addEntry("regexp", value);
            return true;
        case 1:
            o.setDeep(value);
            return false;
        case 2:
            o.setForce(value);
            return false;
        default:
            return false;
        }
    }
    return false;
}
YaripContentWhitelist.prototype.get = function(row, col)
{
    if (!this.sorted) this.sort();
    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        switch (col) {
        case -1: return o.getKey();
        case 0: return o.getRegExp();
        case 1: return o.getDeep();
        case 2: return o.getForce();
        case 3:
            var ms = o.getCreated();
            if (ms > -1) {
                var date = new Date(ms);
                return date.toDateString() + " " + date.toLocaleTimeString();
            } else {
                return "";
            }
        case 4:
            var ms = o.getLastFound();
            if (ms > -1) {
                var date = new Date(ms);
                return date.toDateString() + " " + date.toLocaleTimeString();
            } else {
                return "";
            }
        default: return "";
        }
    }
    return "";
}

function YaripContentBlacklist(name)
{
    this.name = "blacklist";
    this.obj = {};
    this.listLength = 0;
    this.sorted = true;

    this.setName(name);
}
YaripContentBlacklist.prototype = new YaripList;
YaripContentBlacklist.prototype.constructor = YaripContentBlacklist;
YaripContentBlacklist.prototype.set = function(row, col, value)
{
    if (value in this.obj) return;

    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        switch (col) {
        case 0:
            if (!this.checkRegExp(value)) return false;

            var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
            fh.removeEntry("regexp", o.getRegExp());
            this.remove(o);

            o.setRegExp(value);
            this.add(o);
            fh.addEntry("regexp", value);
            return true;
        case 1:
            o.setDeep(value);
            return false;
        case 2:
            o.setForce(value);
            return false;
        default:
            return false;
        }
    }
    return false;
}
YaripContentBlacklist.prototype.get = function(row, col)
{
    if (!this.sorted) this.sort();
    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        switch (col) {
        case -1: return o.getKey();
        case 0: return o.getRegExp();
        case 1: return o.getDeep();
        case 2: return o.getForce();
        case 3:
            var ms = o.getCreated();
            if (ms > -1) {
                var date = new Date(ms);
                return date.toDateString() + " " + date.toLocaleTimeString();
            } else {
                return "";
            }
        case 4:
            var ms = o.getLastFound();
            if (ms > -1) {
                var date = new Date(ms);
                return date.toDateString() + " " + date.toLocaleTimeString();
            } else {
                return "";
            }
        case 5: return o.getIgnored();
        default: return "";
        }
    }
    return "";
}

function YaripPageStyleList(name)
{
    this.name = "style";
    this.obj = "";
    this.listLength = 0;
    this.lastChanged = -1;

    this.setName(name);
}
YaripPageStyleList.prototype = new YaripList;
YaripPageStyleList.prototype.constructor = YaripPageStyleList;
YaripPageStyleList.prototype.set = function(row, col, value)
{
    if (value && value != "") {
        this.obj = value;
        this.listLength = 1;
        this.lastChanged = Date.now();
    } else {
        this.obj = "";
        this.listLength = 0;
        this.lastChanged = -1;
    }
    return false;
}
YaripPageStyleList.prototype.get = function(row, col)
{
    return this.obj;
}
YaripPageStyleList.prototype.clone = function()
{
    var tmp = new this.constructor(this.name);
    tmp.set(null, null, this.get());
    return tmp;
}
YaripPageStyleList.prototype.merge = function(o)
{
    if (o.lastChanged > this.lastChanged) {
        this.set(null, null, o.get());
    }
}
YaripPageStyleList.prototype.subtract = function(o)
{
}
YaripPageStyleList.prototype.generateXml = function()
{
    if (this.listLength > 0) {
        return "\t\t\t<" + this.name + (this.lastChanged > -1 ? " lastChanged=\"" + this.lastChanged + "\"" : "") + "><![CDATA[" + this.get() + "]]></" + this.name + ">\n";
    } else {
        return "";
    }
}

function YaripPageScriptList(name)
{
    this.name = "script";
    this.obj = "";
    this.listLength = 0;
    this.lastChanged = -1;

    this.setName(name);
}
YaripPageScriptList.prototype = new YaripList;
YaripPageScriptList.prototype.constructor = YaripPageScriptList;
YaripPageScriptList.prototype.set = function(row, col, value)
{
    if (value && value != "")
    {
        this.obj = value;
        this.listLength = 1;
        this.lastChanged = Date.now();
    } else {
        this.obj = "";
        this.listLength = 0;
        this.lastChanged = -1;
    }
    return false;
}
YaripPageScriptList.prototype.get = function(row, col)
{
    return this.obj;
}
YaripPageScriptList.prototype.clone = function()
{
    var tmp = new this.constructor(this.name);
    tmp.set(null, null, this.get());
    return tmp;
}
YaripPageScriptList.prototype.merge = function(o)
{
    if (o.lastChanged > this.lastChanged) {
        this.set(null, null, o.get());
    }
}
YaripPageScriptList.prototype.subtract = function(o)
{
}
YaripPageScriptList.prototype.generateXml = function()
{
    if (this.listLength > 0) {
        return "\t\t\t<" + this.name + (this.lastChanged > -1 ? " lastChanged=\"" + this.lastChanged + "\"" : "") + "><![CDATA[" + this.get() + "]]></" + this.name + ">\n";
    } else {
        return "";
    }
}

function YaripHeaderList(name)
{
    this.name = "header";
    this.obj = {};
    this.listLength = 0;
    this.sorted = true;

    this.setName(name);
}
YaripHeaderList.prototype = new YaripList;
YaripHeaderList.prototype.constructor = YaripHeaderList;
YaripHeaderList.prototype.set = function(row, col, value)
{
    if (value in this.obj) return;

    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        switch (col) {
        case 0:
            if (!this.checkRegExp(value)) return false;

            var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
            fh.removeEntry("regexp", o.getRegExp());
            this.remove(o);

            o.setRegExp(value);
            this.add(o);
            fh.addEntry("regexp", value);
            return true;
        case 1:
            var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
//            fh.removeEntry("header_name", o.getRegExp());
            this.remove(o);

            o.setHeaderName(value);
            this.add(o);
            fh.addEntry("header_name", value);
            return true;
        case 2:
            o.setMerge(value);
            return false;
        case 3:
            o.setDeep(value);
            return false;
        case 99:
            o.setScript(value);
            return false;
        default:
            return false;
        }
    }
    return false;
}
YaripHeaderList.prototype.get = function(row, col)
{
    if (!this.sorted) this.sort();
    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        switch (col) {
        case -1: return o.getKey();
        case 0: return o.getRegExp();
        case 1: return o.getHeaderName();
        case 2: return o.getMerge();
        case 3: return o.getDeep();
        case 4:
            var ms = o.getCreated();
            if (ms > -1) {
                var date = new Date(ms);
                return date.toDateString() + " " + date.toLocaleTimeString();
            } else {
                return "";
            }
        case 5:
            var ms = o.getLastFound();
            if (ms > -1) {
                var date = new Date(ms);
                return date.toDateString() + " " + date.toLocaleTimeString();
            } else {
                return "";
            }
        case 99: return o.getScript();
        default: return "";
        }
    }
    return "";
}
YaripHeaderList.prototype.generateXml = function()
{
    var tmp = "";
    for each (var o in this.obj) if (o) tmp += o.generateXml();
    if (tmp != "") {
        return "\t\t\t\t" +
            "<" + this.name + ">\n" +
            tmp +
            "\t\t\t\t</" + this.name + ">\n";
    } else {
        return "";
    }
}

function YaripRedirectList(name)
{
    this.name = "redirect";
    this.obj = {};
    this.listLength = 0;
    this.sorted = true;

    this.setName(name);
}
YaripRedirectList.prototype = new YaripList;
YaripRedirectList.prototype.constructor = YaripRedirectList;
YaripRedirectList.prototype.set = function(row, col, value)
{
    if (value in this.obj) return;

    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        switch (col) {
        case 0:
            if (!this.checkRegExp(value)) return false;

            var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
            fh.removeEntry("regexp", o.getRegExp());
            this.remove(o);

            o.setRegExp(value);
            this.add(o);
            fh.addEntry("regexp", value);
            return true;
        case 1:
            var fh = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2);
            fh.removeEntry("newsubstr", o.getRegExp());

            o.setNewSubStr(value);
            fh.addEntry("newsubstr", value);
            return false;
        case 2:
            o.setDeep(value);
            return false;
        default:
            return false;
        }
    }
    return false;
}
YaripRedirectList.prototype.get = function(row, col)
{
    if (!this.sorted) this.sort();
    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        switch (col) {
        case -1: return o.getKey();
        case 0: return o.getRegExp();
        case 1: return o.getNewSubStr();
        case 2: return o.getDeep();
        case 3:
            var ms = o.getCreated();
            if (ms > -1) {
                var date = new Date(ms);
                return date.toDateString() + " " + date.toLocaleTimeString();
            } else {
                return "";
            }
        case 4:
            var ms = o.getLastFound();
            if (ms > -1) {
                var date = new Date(ms);
                return date.toDateString() + " " + date.toLocaleTimeString();
            } else {
                return "";
            }
        default: return "";
        }
    }
    return "";
}

function YaripPageExtensionList(name)
{
    this.id = null;
    this.name = "extension";
    this.obj = {};
//    this.listLength = 0;
    this.sorted = true;

    this.setName(name);
}
YaripPageExtensionList.prototype = new YaripList;
YaripPageExtensionList.prototype.constructor = YaripPageExtensionList;
YaripPageExtensionList.prototype.set = function(row, col, value)
{
    if (this.obj[value]) return;

    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        switch (col) {
        case 1:
            o.setDoElements(value);
            break;
        case 2:
            o.setDoContents(value);
            break;
        case 3:
            o.setDoScripts(value);
            break;
        case 4:
            o.setDoHeaders(value);
            break;
        case 5:
            o.setDoRedirects(value);
            break;
        case 6:
            o.setDoLinks(value);
            break;
        default:
            break;
        }
    }
    return false;
}
YaripPageExtensionList.prototype.get = function(row, col)
{
    if (!this.sorted) this.sort();
    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        switch (col) {
        case -1: return o.getKey();
        case 0: return o.getPage().getName();
        case 1: return o.getDoElements();
        case 2: return o.getDoContents();
        case 3: return o.getDoScripts();
        case 4: return o.getDoHeaders();
        case 5: return o.getDoRedirects();
        case 6: return o.getDoLinks();
        case 7:
            var ms = o.getCreated();
            if (ms > -1) {
                var date = new Date(ms);
                return date.toDateString() + " " + date.toLocaleTimeString();
            } else {
                return "";
            }
        default: return "";
        }
    }
    return "";
}
YaripPageExtensionList.prototype.__defineGetter__("length", function() {
//        return this.listLength;
        var l = 0;
        for (var k in this.obj) { l++; }
        return l;
    });
YaripPageExtensionList.prototype.add = function(o, purge, tmp)
{
    if (!o) return;

    var page = o.getPage();
    if (!page) return;

    var oldObj = this.obj[o.getKey()];
    if (oldObj) {
        oldObj.merge(o);
        if (purge) oldObj.purge();
    } else {
        if (purge) o.purge();
        this.obj[o.getKey()] = o;
        this.sorted = false;
//        this.listLength++;
    }

    if (!tmp) page.pageExtendedByList.add(new YaripPageExtensionItem(this.id));
}
YaripPageExtensionList.prototype.setId = function(id)
{
    this.id = id;
}
YaripPageExtensionList.prototype.removeByKey = function(value)
{
    if (!value || !value in this.obj) return;
    delete this.obj[value];
    var page = this.getPageById(value);
    if (page) {
        page.pageExtendedByList.removeByKey(this.id, true);
    }
}
YaripPageExtensionList.prototype.clone = function(purge)
{
    var tmp = new this.constructor(this.name);
    tmp.id = this.id;
    for each (var o in this.obj) if (o) tmp.add(o.clone(purge));
    return tmp;
}

function YaripPageExtendedByList()
{
    this.id = null;
    this.obj = {};
//    this.listLength = 0;
    this.sorted = true;
}
YaripPageExtendedByList.prototype = new YaripList;
YaripPageExtendedByList.prototype.constructor = YaripPageExtendedByList;
YaripPageExtendedByList.prototype.set = function(row, col, value)
{
    if (this.obj[value]) return;

    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        var page = o.getPage();
        var o2 = page.pageExtensionList.obj[this.id];

        switch (col) {
        case 1:
            o2.setDoElements(value);
            break;
        case 2:
            o2.setDoContents(value);
            break;
        case 3:
            o2.setDoScripts(value);
            break;
        case 4:
            o2.setDoHeaders(value);
            break;
        case 5:
            o2.setDoRedirects(value);
            break;
        case 6:
            o2.setDoLinks(value);
            break;
        default:
            break;
        }
    }
    return false;
}
YaripPageExtendedByList.prototype.get = function(row, col)
{
    if (!this.sorted) this.sort();

    var i = 0;
    for each (var o in this.obj)
    {
        if (i++ != row) continue;

        var page = o.getPage();
        var o2 = page.pageExtensionList.obj[this.id];

        switch (col) {
        case -1: return o.getKey();
        case 0: return page.getName();
        case 1: return o2.getDoElements();
        case 2: return o2.getDoContents();
        case 3: return o2.getDoScripts();
        case 4: return o2.getDoHeaders();
        case 5: return o2.getDoRedirects();
        case 6: return o2.getDoLinks();
        case 7:
            var ms = o2.getCreated();
            if (ms > -1) {
                var date = new Date(ms);
                return date.toDateString() + " " + date.toLocaleTimeString();
            } else {
                return "";
            }
        default: return "";
        }
    }
    return "";
}
YaripPageExtendedByList.prototype.__defineGetter__("length", function() {
//        return this.listLength;
        var l = 0;
        for (var k in this.obj) { l++; }
        return l;
    });
YaripPageExtendedByList.prototype.setId = function(id)
{
    this.id = id;
}
YaripPageExtendedByList.prototype.removeByKey = function(value)
{
    if (!value || !value in this.obj) return;
    delete this.obj[value];
}
YaripPageExtendedByList.prototype.reset = function()
{
    this.obj = {};
//    this.listLength = 0;
    this.sorted = true;
}
YaripPageExtendedByList.prototype.clone = function(purge)
{
    var tmp = new this.constructor();
    tmp.id = this.id;
    for each (var o in this.obj) if (o) tmp.add(o.clone(purge));
    return tmp;
}
YaripPageExtendedByList.prototype.merge = function(list)
{
    for each (var o in list.obj) {
        var page = o.getPage();
        if (page) {
            page.pageExtensionList.removeByKey(list.id);
            page.pageExtensionList.add(new YaripPageExtensionItem(this.id));
            this.add(o.clone());
        }
    }
}

