
/*
Copyright 2007-2011 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripLoader()
{
    this.hasWhitelist = false;
    this.hasBlacklist = false;
    this.hasStyles = false;
    this.hasScripts = false;

    this.load = function(doc)
    {
        if (!doc) return false;

        var pageName = yarip.getPageName(doc.location, yarip.MODE_PAGE);
        if (!pageName) return false;

        var addressObj = yarip.getAddressObj(pageName, true);
        if (!addressObj.found) return false;

        this.doWhitelisting(doc, pageName, addressObj);
        this.doBlacklisting(doc, pageName, addressObj);
        this.doStyling(doc, pageName, addressObj);
        this.doExtensioning(doc, pageName, addressObj);
        this.doScripting(doc, pageName, addressObj);

        if (this.hasWhitelist && !this.hasBlacklist) yarip.removeAllExceptWhitelisted(doc, pageName);

        var found = this.hasWhitelist || this.hasBlacklist || this.hasStyles || this.hasScripts;
        this.hasWhitelist = false;
        this.hasBlacklist = false;
        this.hasStyles = false;
        this.hasScripts = false;
        return found;
    }

    this.doWhitelisting = function(doc, pageName, addressObj)
    {
        for (var a in addressObj.obj)
        {
            if (!addressObj.obj[a][1]) continue; // not elements

            var isSelf = addressObj.obj[a][0];

            var list = yarip.map.get(a).elementWhitelist.obj;
            for each (var element in list)
            {
                if (isSelf && !element.getDeep() && !yarip.equalAddresses(pageName, a)) {
                    continue;
                }
//                if (yarip.whitelistXPath(doc, a, element.getXPath(), false, null, addressObj.obj[a][0])) {
                if (yarip.whitelistXPath(doc, a, element.getXPath(), false, element.getForce(), null, addressObj.obj[a][0])) {
                    this.hasWhitelist = true;
                }
            }
        }
    }

    this.doBlacklisting = function(doc, pageName, addressObj)
    {
        for (var a in addressObj.obj)
        {
            if (!addressObj.obj[a][1]) continue; // not elements

            var isSelf = addressObj.obj[a][0];

            var list = yarip.map.get(a).elementBlacklist.obj;
            for each (var element in list)
            {
//                if (!element.getForce() && !element.getDeep() && !yarip.equalAddresses(pageName, a)) continue;
                if (isSelf && !element.getDeep() && !yarip.equalAddresses(pageName, a)) {
                    continue;
                }
                if (!this.hasWhitelist && element.getDOW()) {
                    continue;
                }
                if (yarip.blacklistXPath(doc, a, element.getXPath(), false, element.getForce(), element.getPlaceholder(), null, addressObj.obj[a][0])) {
                    this.hasBlacklist = true;
                }
            }
        }
    }

    this.doStyling = function(doc, pageName, addressObj)
    {
        for (var a in addressObj.obj)
        {
            if (!addressObj.obj[a][1]) continue; // not elements

            var page = yarip.map.get(a);

            // page styling
            var list = page.pageStyleList;
            if (list.length > 0)
            {
                var id = "yarip-style-page_" + a.replace(/\W/g, "_");
                var style = doc.getElementById(id);
                if (style) continue;

                this.injectCascadingStyleSheets(doc, id, list.get());
            }

            var isSelf = addressObj.obj[a][0];

            // element styling
            list = page.elementAttributeList.obj;
            for each (var element in list)
            {
                if (isSelf && !element.getDeep() && !yarip.equalAddresses(pageName, a)) {
                    continue;
                }
                if (yarip.styleElement(doc, a, element.getXPath(), element.getName(), element.getValue(), null, null, isSelf)) {
                    this.hasStyles = true;
                }
            }
        }
    }

    this.doExtensioning = function(doc, pageName, addressObj)
    {
        var extAddressObj = yarip.getExtensionAddressObj(addressObj);
        if (!extAddressObj) return;

        this.doWhitelisting(doc, pageName, extAddressObj);
        this.doBlacklisting(doc, pageName, extAddressObj);
        this.doStyling(doc, pageName, extAddressObj);
        this.doScripting(doc, pageName, extAddressObj);
    }

    this.doScripting = function(doc, pageName, addressObj)
    {
        var arr = [];
        for (var a in addressObj.obj) {
            if (!addressObj.obj[a][3]) continue; // not scripts
            arr.push({"a": a, "isSelf": addressObj.obj[a][0]});
        }

        var id = null;
        var script = null;
        var obj = {}
        var found = false;

        // page scripting
        for (var i = arr.length; i > 0; i--)
        {
            var a = arr[i - 1].a;
            var list = yarip.map.get(a).pageScriptList;
            if (list.length <= 0) continue;

            id = "yarip-script-page_" + a.replace(/\W/g, "_");
            script = doc.getElementById(id);
            // TODO Comment out after having added the rule-parameter `reinject' (reinject on DOMNodeInserted)!
            if (script) continue;

//            this.injectJavaScript(doc, id, list.get());
            obj[id] = list.get();
            found = true;
        }

        // element scripting
        // TODO Comment out after having added the rule-parameter `reinject' (reinject on DOMNodeInserted)!
        script = doc.getElementById("yarip-script-element");
        if (script) return;

        for (var i = arr.length; i > 0; i--)
        {
            var aObj = arr[i - 1];
            var a = aObj.a;
            var list = yarip.map.get(a).elementScriptList;
            if (list.length == 0) continue;

            var scriptIdPrefix = "yarip-script-element_" + a.replace(/\W/g, "_") + "_";
            var count = 0;

            for each (var o in list.obj)
            {
                if (aObj.isSelf && !o.getDeep() && !yarip.equalAddresses(pageName, a)) {
                    continue;
                }

                var s = o.getScript();
                if (!s) continue;

                id = scriptIdPrefix + count;
                script = doc.getElementById(id);
                // TODO Comment out after having added the rule-parameter `reinject' (reinject on DOMNodeInserted)!
                if (script) {
                    count++;
                    continue;
                }

                // TODO How to evaluate the XPath only once!?
                if (aObj.isSelf) {
                    var elements = yarip.getElementsByXPath(doc, o.getXPath());
                    if (elements && elements.snapshotLength > 0) {
                        o.incrementFound();
                    } else {
                        o.incrementNotFound();
                        continue;
                    }
                }

                var x = o.getXPath().replace(/(["])/g, "\\$1"); // TODO Anything else!?
                obj[id] = "yarip.run(" + s + ",\"" + x + "\");\n";
                count++;
                found = true;
            }
        }

        if (found)
        {
            this.injectJavaScript(doc, "yarip-script-element", "" +
                    "if (!yarip) {\n" +
                    "  var yarip = {\n" +
                    "    $: function(x) {\n" +
                    "      var arr = [];\n" +
                    "      var xr = document.evaluate(x, document, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);\n" +
                    "      if (xr) for (var i = 0; i < xr.snapshotLength; i++) arr.push(xr.snapshotItem(i));\n" +
                    "      return arr;\n" +
                    "    },\n" +
                    "    run: function(fun, x) {\n" +
                    "      fun.call(this, this.$(x));\n" +
                    "    }\n" +
                    "  }\n" +
                    "}");
            for (var id in obj) {
                this.injectJavaScript(doc, id, obj[id]);
            }
        }
    }

    this.injectCascadingStyleSheets = function(doc, id, css)
    {
        var style = doc.createElement("style");
        style.type = "text/css";
        style.setAttribute("id", id);
        style.setAttribute("status", "whitelisted");
        style.innerHTML = css;
        doc.getElementsByTagName("head")[0].appendChild(style);
        this.hasStyles = true;
    }

    this.injectJavaScript = function(doc, id, js)
    {
        var e = doc.getElementById(id);
        if (e) e.parentNode.removeChild(e);
        var script = doc.createElement("script");
        script.type = "text/javascript";
        script.setAttribute("id", id);
        script.setAttribute("status", "whitelisted");
        script.innerHTML = js;
        doc.getElementsByTagName("head")[0].appendChild(script);
        this.hasScripts = true;
    }
}
