
/*
Copyright 2007-2011 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// TODO Create a `cleanup'-function which deletes page-references for non-existent pages!
function YaripMap()
{
    this.obj = {};
    this.objId = {};
    this.tree = new YaripAATree();
    this.length = 0;
    this.version = this.currentVersion();

    this.add = function(page)
    {
        if (!page) return;

        var oldPage = this.obj[page.getName()];
        if (oldPage) {
            oldPage.merge(page);
        } else {
            this.obj[page.getName()] = page;
            this.objId[page.getId()] = page;
            this.tree.add(page);
            this.length++;
        }
    }

    this.remove = function(page)
    {
        if (!page) return;

        var pageId = page.getId();
        if (pageId) {
            this.removeById(page.getId());
        } else {
            this.removeByName(page.getName());
        }
    }

    this.get = function(pageName)
    {
        return pageName in this.obj ? this.obj[pageName] : null;
//        return this.obj[pageName];
    }

    this.getById = function(value)
    {
        return value in this.objId ? this.objId[value] : null;
    }

    this.removeById = function(id)
    {
        if (!id || !this.objId[id]) return;
        var page = this.objId[id];
        this.tree.remove(page);
        delete this.obj[page.getName()];
        delete this.objId[id];
        this.length--;
    }

    this.removeByName = function(pageName)
    {
        if (!pageName || !this.obj[pageName]) return;
        var page = this.obj[pageName];
        this.tree.remove(page);
        delete this.objId[page.getId()];
        delete this.obj[pageName];
        this.length--;
    }

    this.clone = function()
    {
        var map = new this.constructor();
        for each (var page in this.obj) if (page) map.add(page.clone());
        return map;
    }

//    this.replaceExtensionIds = function(map, newId, oldId)
//    {
//        if (!map || !newId || !oldId) return;
//        var mapObj = map.obj;
//        for each (var page in mapObj)
//        {
//            var list = page.pageExtensionList;
//            var listObj = list.obj;
//            for each (var oldItem in listObj)
//            {
//                if (oldItem.getValue() == oldId)
//                {
//                    var newItem = oldItem.clone();
//                    newItem.setValue(newId);
//                    list.remove(oldItem);
//                    list.add(newItem);
//                }
//            }
//        }
//    }
    this.replaceExtensionIds = function(map, oldPage, newId)
    {
        if (!map || !oldPage || !newId) return;

        map.remove(oldPage);
        var newPage = oldPage.clone(null, null, newId);
        map.add(newPage);
    }

    this.merge = function(map)
    {
        if (!map) return;
        var mapObj = map.obj;
        for (var p in mapObj)
        {
            var page = map.obj[p];
            if (this.obj[p]) { // same name
                this.replaceExtensionIds(map, page, this.obj[p].getId()); // use existing id
            } else if (this.objId[page.getId()]) { // same id
                var newId = null;
                do {
                    newId = this.newId();
                } while(this.getById(newId) != undefined && this.getById(newId) != null
                        && map.getById(newId) != undefined && map.getById(newId) != null);
                this.replaceExtensionIds(map, page, newId); // use new id
            }
        }
        for each (var page in mapObj) {
            this.add(page);
        }
    }

    this.purge = function()
    {
        for each (var page in this.obj) if (page) page.purge();
    }

    this.generateXml = function(exporting)
    {
        var r = "<?xml version=\"1.0\" encoding=\"" + this.CHARSET + "\"?>\n";
        if (!exporting) {
            r += "\n<!--\nDo not edit this file.\n\nIf you make changes to this file while the application is running,\nthe changes will be overwritten when the application exits.\n-->\n\n";
        }
        r += "<yarip version=\"" + this.version + "\">\n";
        this.tree.traverse(function(node) { if (node.value) r += node.value.generateXml(); });
        r += "</yarip>";
        return r;
    }
}
YaripMap.prototype = new YaripObject;
YaripMap.prototype.constructor = YaripMap;

