
/*
Copyright 2007-2011 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful;
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

var EXPORTED_SYMBOLS = [
    "VERSION",
    "FILE",
    "CHARSET",

    "MODE_PAGE",
    "MODE_FQDN",
    "MODE_SLD",

    "TYPE_WHITELIST",
    "TYPE_BLACKLIST",
    "TYPE_ATTRIBUTE",

    "DATA_TYPE_BOOLEAN",
    "DATA_TYPE_INTEGER",
    "DATA_TYPE_STRING",
    "DATA_TYPE_RESET",

    "INCREMENT_NOT_FOUND",
    "INCREMENT_IGNORED",
    "INCREMENT_FOUND",

    "STATUS_UNKNOWN",
    "STATUS_WHITELISTED",
    "STATUS_BLACKLISTED",
    "STATUS_REDIRECTED",

    "PREF_VERSION",
    "PREF_ENABLED",
    "PREF_FLICKER",
    "PREF_MODE",
    "PREF_INDEX",
    "PREF_ELEMENTS",
    "PREF_EXCLUSIVE",
    "PREF_TEMPLATES",
    "PREF_PRIVATE",
    "PREF_PURGE",
    "PREF_SCHEMES",
//    "PREF_REPEAT",
    "PREF_RECURRENCE",

    "urlRegExp",
    "uriSimpleRegExp",
    "noSchemeRegExp",
    "domainRegExp",
    "findSldRegExp",
    "findTldRegExp",
    "outlineRegExp",
    "endOutlineRegExp",

    "mode",
    "enabled",
    "noFlicker",

    "monitorDialogues",
    "pageDialog",

    "alwaysUseIndex",
    "elementsInContext",
    "exclusiveOnCreation",
    "templates",
    "privateBrowsing",
    "purgeInnerHTML",
    "schemesRegExp",
//    "contentRepeatThreshold",
    "contentRecurrence",

    "map",
    "undoObj",
    "pageCreated",
    "knownAddressObj",

    "getLocationFromLocation",
    "getContentLocationFromContentLocation",
    "getLocationFromContentLocation",
    "getPageName"
];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
const Cu = Components.utils;

var SSL = Cc["@mozilla.org/moz/jssubscript-loader;1"].getService(Ci.mozIJSSubScriptLoader);
var IOS = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
var PB = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
var CS = Cc["@mozilla.org/consoleservice;1"].getService(Ci.nsIConsoleService);
var OS = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
var IDNS = Cc["@mozilla.org/network/idn-service;1"].getService(Ci.nsIIDNService);
var WP = Cc['@mozilla.org/docloaderservice;1'].getService(Ci.nsIWebProgress);

Cu.import("resource://yarip/uri.jsm");

var VERSION = "0.2.6";
var FILE = "yarip.xml";
var CHARSET = "UTF-8";

var MODE_PAGE = 0;
var MODE_FQDN = 1;
var MODE_SLD = 2;

var TYPE_WHITELIST = 0;
var TYPE_BLACKLIST = 1;
var TYPE_ATTRIBUTE = 2;

var DATA_TYPE_BOOLEAN = 0;
var DATA_TYPE_INTEGER = 1;
var DATA_TYPE_STRING = 2;
var DATA_TYPE_RESET = 3;

var INCREMENT_NOT_FOUND = 0;
var INCREMENT_IGNORED = 1;
var INCREMENT_FOUND = 2;

var STATUS_UNKNOWN = 0;
var STATUS_WHITELISTED = 1;
var STATUS_BLACKLISTED = 2;
var STATUS_REDIRECTED = 3;

var PREF_VERSION = "extensions.yarip.version";
var PREF_ENABLED = "extensions.yarip.enabled.value";
var PREF_FLICKER = "extensions.yarip.noFlicker.value";
var PREF_MODE = "extensions.yarip.mode.value";
var PREF_INDEX = "extensions.yarip.alwaysUseIndex.value";
var PREF_ELEMENTS = "extensions.yarip.elementsInContext.value";
var PREF_EXCLUSIVE = "extensions.yarip.exclusiveOnCreation.value";
var PREF_TEMPLATES = "extensions.yarip.templatesList.value";
var PREF_PRIVATE = "extensions.yarip.privateBrowsing.value";
var PREF_PURGE = "extensions.yarip.purgeInnerHTML.value";
var PREF_SCHEMES = "extensions.yarip.schemesRegExp.value";
//var PREF_REPEAT = "extensions.yarip.contentRepeatThreshold.value";
var PREF_RECURRENCE = "extensions.yarip.contentRecurrence.value";

var urlRegExp = URL_RE;
var uriSimpleRegExp = URI_SIMPLE_RE;
var noSchemeRegExp = NO_SCHEME_RE;
var domainRegExp = DOMAIN_RE;
var findSldRegExp = FIND_SLD_RE;
var findTldRegExp = FIND_TLD_RE;
var outlineRegExp = /^outline-color:\ rgb\((0|153),\ 0,\ (0|153)\);\ outline-style:\ solid;\ outline-width:\ 3px;$/;
var endOutlineRegExp = /outline-color:\ rgb\((0|153),\ 0,\ (0|153)\);\ outline-style:\ solid;\ outline-width:\ 3px;$/;

var mode = null;
var enabled = null;
var noFlicker = null;

var monitorDialogues = [];
var pageDialog = null;

var alwaysUseIndex = false;
var elementsInContext = 4;
var exclusiveOnCreation = false;
var templates = [];
var privateBrowsing = false;
var purgeInnerHTML = false;
var schemesRegExp = null;
var contentRepeatThreshold = 10000;
//var contentRecurrence = false;

var map = null;
var undoObj = {};
var pageCreated = false;
var knownAddressObj = {};

function getLocationFromLocation(location)
{
    if (!location) return null;
    if (location.isLocation) return location;

    // https://developer.mozilla.org/en/DOM/window.location
    var host = "";
    var asciiHost = "";
//    var hostname = "";
    var port = location.port;
    var asciiHref = "";
    var pageName = "";
    try {
        host = location.host;
        asciiHost = IDNS.convertUTF8toACE(host);
//        hostname = IDNS.convertUTF8toACE(location.hostname);
//        asciiHref = location.protocol + "//" + asciiHost + location.pathname;
        pageName = location.protocol + "//" + asciiHost + location.pathname;
//        asciiHref = location.protocol + "//" + asciiHost + location.pathname + location.search + location.hash;
        asciiHref = pageName + location.search + location.hash;
//        asciiHref = IDNS.convertUTF8toACE(location.href);
    } catch(e) {}
    return {
//        hash: null,
        host: host,
//        hostname: null,
//        href: protocol + "//" + host + pathname, // no need for search nor hash!
        href: location.href,
        pathname: location.pathname,
        port: port,
        protocol: location.protocol,
//        search: null,
        asciiHost: asciiHost,
        asciiHref: asciiHref,
        pageName: pageName.replace(/[/]*$/, ""),
        isLocation: true
    }
}

function getLocationFromContentLocation(contentLocation)
{
    if (!contentLocation) return null;
    if (contentLocation.isLocation) return contentLocation;

    // https://developer.mozilla.org/en/nsIURI
    // https://developer.mozilla.org/en/DOM/window.location
    var host = "";
    try { host = contentLocation.hostPort; } catch (e) {}
//    try { host = contentLocation.host; } catch (e) {}
    var pathname = contentLocation.path.replace(/[?&#].*$/, "");
    var port = "";
    try { port = contentLocation.port; } catch (e) {}
    var protocol = contentLocation.scheme + ":";
    var asciiHost = "";
    try { asciiHost = contentLocation.asciiHost + (port > -1 && port != 80 ? ":" + port : ""); } catch (e) {}
    var pageName = "";
    return {
//        hash: null,
        host: host,
//        hostname: null,
        href: contentLocation.spec,
        pathname: pathname,
        port: port,
        protocol: protocol,
//        search: null,
//        asciiHost: IDNS.convertUTF8toACE(host),
        asciiHost: asciiHost,
        asciiHref: contentLocation.asciiSpec,
        pageName: protocol + "//" + asciiHost + pathname.replace(/[/]*$/, ""),
        isLocation: true
    }
}

function getContentLocationFromContentLocation(contentLocation)
{
    if (!contentLocation) return null;
    if (contentLocation.isContentLocation) return contentLocation;

    // https://developer.mozilla.org/en/nsIURI
    var port = "";
    var host = "";
    var asciiHost = "";
    var hostPort = "";
    try { port = contentLocation.port; } catch (e) {}
    try { host = contentLocation.host; } catch (e) {}
    try { asciiHost = contentLocation.asciiHost; } catch (e) {}
    try { hostPort = contentLocation.hostPort; } catch (e) {}
    return {
        scheme: contentLocation.scheme,
        port: port,
        host: host,
        asciiHost: asciiHost,
        hostPort: hostPort,
        path: contentLocation.path,
        spec: contentLocation.spec,
        asciiSpec: contentLocation.asciiSpec,
        isContentLocation: true
    }
}

function getPageName(location, mode)
{
//    if (!location || !/^https?:$/.test(location.protocol)) return null;
    if (!location || !this.schemesRegExp.test(location.protocol.replace(/:$/, ""))) return null;

    var mode = Number(mode);
    if (typeof mode != "number" || mode + "" == "NaN") {
        mode = this.mode;
    }

    location = this.getLocationFromLocation(location);
    var address = null;
    try
    {
        switch (mode) {
        case this.MODE_PAGE:
//            address = location.protocol + "//" + location.host + location.pathname.replace(/[/]*$/, "");
            address = location.protocol + "//" + location.asciiHost + location.pathname.replace(/[/]*$/, "");
//            address = location.asciiHref.replace(/[/]*$/, "");
            break;
        case this.MODE_FQDN:
            address = location.asciiHost;
            break;
        case this.MODE_SLD:
            var sld = null;
            sld = location.asciiHost.match(this.findSldRegExp);
            if (sld) address = sld[0];
            break;
        }
        if (!address) return null;
    } catch (e) {}
    return address;
}

